#include <string.h>
#include <gccore.h>
#include <malloc.h>

#include "..\include\sys.h"
#include "..\include\video.h"

#include "ticket_dat.h"
#include "tmd_dat.h"

// Certificates
static u8 certs[0xA00] ATTRIBUTE_ALIGN(32);

bool GetCertificates(bool isFrench)
{ 
	ISFS_Initialize();

	int fd = ISFS_Open("/sys/cert.sys", ISFS_OPEN_READ);

	if (fd < 0)
	{
		if (isFrench)
			PrintText("  Impossible d'ouvrir le 'ISFS'!\n", false, false, false, WHITE, RED);
		else
			PrintText("  Unable to open ISFS!\n", false, false, false, WHITE, RED);

		return false;
	}

	if (ISFS_Read(fd, certs, sizeof(certs)) != sizeof(certs))
	{
		if (isFrench)
			PrintText("  Impossible de lire le 'ISFS'!\n", false, false, false, WHITE, RED);
		else
			PrintText("  Unable to read ISFS!\n", false, false, false, WHITE, RED);

		return false;
	}

	ISFS_Close(fd);

	return true;
}

// Get the system menu version from TMD
u32 GetSysMenuVersion(bool isFrench)
{
	static u64 TitleID ATTRIBUTE_ALIGN(32) = 0x0000000100000002LL;
	static u32 tmdSize ATTRIBUTE_ALIGN(32);

	// Get the stored TMD size for the system menu
	if (ES_GetStoredTMDSize(TitleID, &tmdSize) < 0)
	{
		if (isFrench)
			PrintText("  Erreur lors de l'appel a ES_GetStoredTMDSize!\n", false, false, false, WHITE, RED);
		else
			PrintText("  Failed to get the stored TMD size for the system menu!\n", false, false, false, WHITE, RED);

		return false;
	}

	signed_blob *TMD = (signed_blob *)memalign(32, (tmdSize+32)&(~31));
	memset(TMD, 0, tmdSize);

	// Get the stored TMD for the system menu
	if (ES_GetStoredTMD(TitleID, TMD, tmdSize) < 0)
	{
		if (isFrench)
			PrintText("  Erreur lors de l'appel a ES_GetStoredTMD!\n", false, false, false, WHITE, RED);
		else
			PrintText("  Failed to get the stored TMD for the system menu!\n", false, false, false, WHITE, RED);

		return false;
	}

	// Get the system menu version from TMD
	tmd *rTMD = (tmd *)(TMD+(0x140/sizeof(tmd *)));
	u32 version = rTMD->title_version;

	free(TMD);

	// Return the system menu version
	return version;
}

// Get the boot2 version
u32 GetBoot2Version(bool isFrench)
{
	u32 boot2version = 0;

	if (ES_GetBoot2Version(&boot2version) < 0)
	{
		if (isFrench)
			PrintText("  Erreur lors de l'appel a ES_GetBoot2Version!\n", false, false, false, WHITE, RED);
		else
			PrintText("  Failed getting the boot2 version!\n", false, false, false, WHITE, RED);

		boot2version = 0;
    }
	
	return boot2version;
}

// Get the console ID
u32 GetDeviceID(bool isFrench)
{
	u32 deviceId = 0;

	if (ES_GetDeviceID(&deviceId) < 0)
	{
		if (isFrench)
			PrintText("  Erreur lors de l'appel a ES_GetDeviceID!\n", false, false, false, WHITE, RED);
		else
			PrintText("  Failed getting the console ID!\n", false, false, false, WHITE, RED);

		deviceId = 0;
    }
	
	return deviceId;
}

// Remove bogus ticket
int RemoveBogusTicket(void)
{
	tikview *viewdata = NULL;
	u64 titleId = 0x100000000LL;
	u32 cnt, views;
	s32 ret;

	// Get number of ticket views
	ret = ES_GetNumTicketViews(titleId, &views);

	if (ret < 0) return ret;

	if (!views) return 1;
	else if (views > 16) return -1;
	
	// Get ticket views
	viewdata = (tikview*)memalign(32, sizeof(tikview) * views);
	ret = ES_GetTicketViews(titleId, viewdata, views);

	if (ret < 0) return ret;

	// Remove tickets
	for (cnt = 0; cnt < views; cnt++)
	{
		ret = ES_DeleteTicket(&viewdata[cnt]);

		if (ret < 0) return ret;
	}

	return ret;
}

// Check fake signatures (aka Trucha Bug)
bool CheckFakeSignature(void)
{
	int ret = ES_AddTicket((signed_blob *)ticket_dat, ticket_dat_size, (signed_blob *)certs, sizeof(certs), 0, 0);

	if (ret > -1) RemoveBogusTicket();
	if (ret > -1 || ret == -1028) return true;

	return false;
}

// Check if you can still call ES_DiVerify (aka ES_Identify) to make IOS think that you actually are a different title
bool CheckESIdentify(void)
{
	int ret = ES_Identify((signed_blob *)certs, sizeof(certs), (signed_blob *)tmd_dat, tmd_dat_size, (signed_blob *)ticket_dat, ticket_dat_size, NULL);
 
	if (ret == -2011) ret = 0;

	if (ret < 0)
		return false;
	else
		return true;
}

// Check flash access
bool CheckFlashAccess(void)
{
	int ret = IOS_Open("/dev/flash", 1);

	if (ret >= 0) IOS_Close(ret);

	if (ret < 0)
		return false;
	else
		return true;
}

// Check NAND access
bool CheckNANDAccess(void)
{
	int ret = IOS_Open("/title/00000001/00000002/content/title.tmd", 1);

	if (ret >= 0) IOS_Close(ret);

	if (ret < 0)
		return false;
	else
		return true;
}

// Check boot2 access
bool CheckBoot2Access(void)
{
	int ret = IOS_Open("/dev/boot2", 1);

	if (ret >= 0) IOS_Close(ret);

	if (ret < 0)
		return false;
	else
		return true;
}

// Check USB 2.0 module
bool CheckUSB2(u32 titleID)
{
	// Hermes' IOS supports USB 2.0 module
    switch (titleID)
	{
		case 202:
			return true;
			break;

		case 222:
			return true;
			break;

		case 223:
			return true;
			break;

		case 224:
			return true;
			break;
    }

	// Open USB 2.0 module
	int ret = IOS_Open("/dev/usb2", 1);
	 
	// If fail, try old USB 2.0 module
	if (ret < 0) ret = IOS_Open("/dev/usb/ehc", 1);

	if (ret >= 0) IOS_Close(ret);

	if (ret < 0)
		return false;
	else
		return true;
}

// Check if this is an IOS stub (according to WiiBrew.org)
bool IsKnownStub(u32 noIOS, s32 noRevision)
{
	if (noIOS ==   3 && noRevision == 65280) return true;
	if (noIOS ==   4 && noRevision == 65280) return true;
	if (noIOS ==  10 && noRevision ==   768) return true;
	if (noIOS ==  11 && noRevision ==   256) return true;
	if (noIOS ==  16 && noRevision ==   512) return true;
	if (noIOS ==  20 && noRevision ==   256) return true;
	if (noIOS ==  30 && noRevision ==  2816) return true;
	if (noIOS ==  40 && noRevision ==  3072) return true;
	if (noIOS ==  50 && noRevision ==  5120) return true;
	if (noIOS ==  51 && noRevision ==  4864) return true;
	if (noIOS ==  52 && noRevision ==  5888) return true;
	if (noIOS ==  60 && noRevision ==  6400) return true;
	if (noIOS ==  70 && noRevision ==  6912) return true;
	if (noIOS == 222 && noRevision == 65280) return true;
	if (noIOS == 223 && noRevision == 65280) return true;
	if (noIOS == 249 && noRevision == 65280) return true;
	if (noIOS == 250 && noRevision == 65280) return true;
	if (noIOS == 254 && noRevision ==     2) return true;
	if (noIOS == 254 && noRevision ==     3) return true;
	if (noIOS == 254 && noRevision ==   260) return true;

// BootMii As IOS is installed on IOS254 rev 31338
	if (noIOS == 254 && noRevision == 31338) return true;

	return false;
}
